"""Auto-generated model classes
"""

# [DO NOT EDIT MANUALLY]

# Auto-generated by Hera via `make events-models`
# OpenAPI spec URL: https://raw.githubusercontent.com/argoproj/argo-workflows/v3.4.4/api/openapi-spec/swagger.json

from hera.events.models.eventsource import (
    CreateEventSourceRequest,
    EventSourceDeletedResponse,
    EventSourceWatchEvent,
    LogEntry as EventsourceLogEntry,
    UpdateEventSourceRequest,
)
from hera.events.models.io.argoproj.events.v1alpha1 import (
    Amount,
    AMQPConsumeConfig,
    AMQPEventSource,
    AMQPExchangeDeclareConfig,
    AMQPQueueBindConfig,
    AMQPQueueDeclareConfig,
    ArgoWorkflowTrigger,
    ArtifactLocation,
    AWSLambdaTrigger,
    AzureEventHubsTrigger,
    AzureEventsHubEventSource,
    Backoff,
    BasicAuth,
    BitbucketAuth,
    BitbucketBasicAuth,
    BitbucketEventSource,
    BitbucketRepository,
    BitbucketServerEventSource,
    BitbucketServerRepository,
    CalendarEventSource,
    CatchupConfiguration,
    Condition,
    ConditionsResetByTime,
    ConditionsResetCriteria,
    ConfigMapPersistence,
    CustomTrigger,
    DataFilter,
    EmitterEventSource,
    EventContext,
    EventDependency,
    EventDependencyFilter,
    EventDependencyTransformer,
    EventPersistence,
    EventSource,
    EventSourceFilter,
    EventSourceList,
    EventSourceSpec,
    EventSourceStatus,
    ExprFilter,
    FileArtifact,
    FileEventSource,
    GenericEventSource,
    GitArtifact,
    GitCreds,
    GithubAppCreds,
    GithubEventSource,
    GitlabEventSource,
    GitRemoteConfig,
    HDFSEventSource,
    HTTPTrigger,
    Int64OrString,
    K8SResourcePolicy,
    KafkaConsumerGroup,
    KafkaEventSource,
    KafkaTrigger,
    LogTrigger,
    Metadata,
    MQTTEventSource,
    NATSAuth,
    NATSEventsSource,
    NATSTrigger,
    NSQEventSource,
    OpenWhiskTrigger,
    OwnedRepositories,
    PayloadField,
    PubSubEventSource,
    PulsarEventSource,
    PulsarTrigger,
    RateLimit,
    RedisEventSource,
    RedisStreamEventSource,
    Resource,
    ResourceEventSource,
    ResourceFilter,
    S3Artifact,
    S3Bucket,
    S3Filter,
    SASLConfig,
    SecureHeader,
    Selector,
    Sensor,
    SensorList,
    SensorSpec,
    SensorStatus,
    Service,
    SlackEventSource,
    SlackTrigger,
    SNSEventSource,
    SQSEventSource,
    StandardK8STrigger,
    Status,
    StatusPolicy,
    StorageGridEventSource,
    StorageGridFilter,
    StripeEventSource,
    Template,
    TimeFilter,
    TLSConfig,
    Trigger,
    TriggerParameter,
    TriggerParameterSource,
    TriggerPolicy,
    TriggerTemplate,
    URLArtifact,
    ValueFromSource,
    WatchPathConfig,
    WebhookContext,
    WebhookEventSource,
)
from hera.events.models.io.argoproj.workflow.v1alpha1 import (
    Event,
    EventResponse,
    GetUserInfoResponse,
    InfoResponse,
    Item,
    Version,
)
from hera.events.models.io.k8s.api.core.v1 import (
    Effect,
    ImagePullPolicy,
    Operator,
    OperatorModel,
    Phase,
    Protocol,
    Scheme,
    TerminationMessagePolicy,
    Type,
    TypeModel,
)
from hera.events.models.sensor import (
    CreateSensorRequest,
    DeleteSensorResponse,
    LogEntry as SensorLogEntry,
    SensorWatchEvent,
    UpdateSensorRequest,
)
