class InvalidType(Exception):
    ...


class InvalidTemplateCall(Exception):
    ...


class InvalidDispatchType(Exception):
    ...


__all__ = ["InvalidType", "InvalidTemplateCall", "InvalidDispatchType"]
